/* Domain to IP Utility
 *
 * Version 1.0
 * 12/24/2020
 *
 * dom.c
 */

#include <arpa/inet.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>

#define PROG_HELP \
"Domain to IP Utility: Get IP address of hostname\n\
\n\
dom [-H] <hostname>\n\
\n\
\t[-H] - Display program syntax and options\n\
\n\
\tIf <hostname> is a valid IP address, it will return itself.\n"

char * get_ip(char hostname[])
{
	static char ip[16];
	struct hostent *ent;
	struct in_addr **addr;

	if ((ent = gethostbyname(hostname)) == NULL)
	{
		return "(not found)";
	}

	addr = (struct in_addr **) ent->h_addr_list;

	strcpy(ip, inet_ntoa(*addr[0]));

	return ip;
}

int main(int argc, char *argv[])
{
	if (argc < 2)
	{
		printf("Usage: %s [-H] <hostname>\n", argv[0]);
		exit(1);
	}

	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 'H')
	{
		printf(PROG_HELP);
		return 0;
	}

	printf("%s\n", get_ip(argv[1]));

	exit(0);
	
}
