/* Date & Time Utility
 *
 * Version 1.0
 * 12/19/2020
 *
 * dt.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define PROG_HELP \
"Date & Time Utility: Display date and/or time\n\
\n\
dt [-H] [-h] [-f] [-F] [-t]\n\
\n\
\t[-H] - Display program syntax and options\n\
\t[-h] - Display additional program help\n\
\t[-f] - Display full date with name of month\n\
\t[-F] - Same as '-f' but includes time\n\
\t[-t] - Display time only\n\
\n\
\tThis program only supports one option at a time.\n"

#define HELP_MESG	"Centaur Date & Time Utility v1.1\nPrints human-readable date and/or time\nDates take the format XX/XX/XXXX; time is formatted as XX:XX [AM|PM]\n\nOptions:\n\t-H -- displays program syntax and options\n\t-h -- displays this help and exits\n\t-f -- displays full date with name of month\n\t-F -- same as -f but includes time\n\t-t -- display time only\n"

#define _USAGE	"Usage: %s [-H] [-h] [-f] [-F] [-t]\n"

#define OPT_HELP	'H'
#define OPT_HELP_	'h'
#define OPT_FULL	'f'
#define OPT_FULLT	'F'
#define OPT_TIME	't'

char option;

int opt(unsigned char ch)
{
	int r = 0;
	
	if (option == ch)
	{
		r = 1;
	}

	return r;
}

int main (int argc, char *argv[])
{
	if (argc >= 2)
	{
		option = argv[1][1];
	} else {
		option = ' ';
	}

	if (opt(OPT_HELP))
	{
		printf(PROG_HELP);
		exit(0);
	}

	time_t rt;
	struct tm *tptr;
	char date[11];
	char fdate[20];
	char ftime[10];

	time(&rt);
	tptr = localtime(&rt);

	strftime(date, 11, "%m/%d/%Y", tptr);
	strftime(fdate, 20, "%B %e, %Y", tptr);
	strftime(ftime, 10, "%I:%M %p", tptr);

	if (argc >= 2 && argv[1][0] == '-' && opt(OPT_HELP_))
	{
		printf("%s\n", HELP_MESG);
		printf(_USAGE, argv[0]);
		exit(0);
	} else if (argc >= 2 && argv[1][0] == '-' && opt(OPT_FULL))
	{
		printf("%s\n", fdate);
		exit(0);
	} else if (argc >= 2 && argv[1][0] == '-' && opt(OPT_FULLT))
	{
		printf("%s %s\n", fdate, ftime);
		exit(0);
	} else if (argc >= 2 && argv[1][0] == '-' && opt(OPT_TIME)) {
		printf("%s\n", ftime);
		exit(0);
	} else if (argc < 2) {
		printf("%s %s\n", date, ftime);
		exit(0);
	} else {
		printf(_USAGE, argv[0]);
		exit(1);
	}
}
