/* File Download Utility
 *
 * Version 1.0
 * 12/24/2020
 *
 * fd.c
 */

#include <curl/curl.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#define PROG_HELP \
"File Download Utility: Download single files\n\
\n\
fd [-H] <url> [output-file]\n\
\n\
\t[-H] - Display program syntax and options\n\
\n\
\tFailed downloads often result in a blank file.\n\
\tOutput file defaults to the file specifed in the URL if not provided.\n"

enum
{
	_NONE,
	_FILE,
	_DIR
};

int ftype(char file[])
{
	struct stat fstat;
	int s = stat(file, &fstat);
	if (s != 0)
	{
		return _NONE;
	}
	if (S_ISREG (fstat.st_mode)) 
	{
		return _FILE;
	} else if (S_ISDIR (fstat.st_mode)) 
	{
		return _DIR;
	} else {
		return _NONE;
	}
}

char * outfile(char url[])
{
	int i;
	int j = 0;
	int k;
	static char _out[56] = "";
	char _out_r[56] = "";
	for (i = strlen(url) - 1; ; i--)
	{
		if (url[i] == '/')
		{
			break;
		}
		_out_r[j] = url[i];
		j++;
	}

	k = strlen(_out_r) - 1;

	for (i = 0; i <= strlen(_out_r) - 1; i++)
	{
		if (_out_r[k] == '\0')
		{
			break;
		}
		_out[i] = _out_r[k];
		k--;
	}

	return _out;
}

int main (int argc, char *argv[])
{
	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 'H')
	{
		printf(PROG_HELP);
		exit(0);
	}

	if (argc < 2 || argc > 3)
	{
		printf("Usage: %s [-H] <url> [output-file]\n", argv[0]);
		exit(1);
	}

	CURLcode cc;
	CURL *hnd;
	char output[56];
	int fd;

	if (argc == 2)
	{
		strcpy(output, outfile(argv[1]));
	} else if (argc == 3) {
		strcpy(output, argv[2]);
	}

	if (!strcmp(output, ""))
	{
		printf("%s: no file specified in URL\n", argv[0]);
		exit(1);
	}

	if (ftype(output) != _NONE)
	{
		printf("%s: local file '%s' already exists\n", argv[0], output);
		exit(1);
	}

	hnd = curl_easy_init();
	curl_easy_setopt(hnd, CURLOPT_URL, argv[1]);
	curl_easy_setopt(hnd, CURLOPT_NOPROGRESS, 1);
	curl_easy_setopt(hnd, CURLOPT_USE_SSL, CURLUSESSL_NONE);
	curl_easy_setopt(hnd, CURLOPT_USERAGENT, "curl/7.22.0");
	curl_easy_setopt(hnd, CURLOPT_MAXREDIRS, 50);

	fd = open(output, O_WRONLY|O_CREAT, 0644);

	close(1);
	dup(fd);

	cc = curl_easy_perform(hnd);

	curl_easy_cleanup(hnd);
	close(fd);
	hnd = NULL;

	exit((int) cc);
}
