/* Read Line Utility
 *
 * Version 1.0
 * 12/27/2020
 *
 * rdln.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#define PROG_HELP \
"Read Line Utility: Read a specific line number from a file\n\
\n\
rdln [-H] <file> <line>\n\
\n\
\t[-H] - Display program syntax and options\n"


enum
{
	_NONE,
	_FILE,
	_DIR
};

int ftype(char file[])
{
	struct stat fstat;
	int s = stat(file, &fstat);
	if (s != 0)
	{
		return _NONE;
	}
	if (S_ISREG (fstat.st_mode)) 
	{
		return _FILE;
	} else if (S_ISDIR (fstat.st_mode)) 
	{
		return _DIR;
	} else {
		return _NONE;
	}
}

int main(int argc, char *argv[])
{
	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 'H')
	{
		printf(PROG_HELP);
		exit(0);
	}

	if (argc != 3)
	{
		printf("Usage: %s [-H] <file> <line>\n", argv[0]);
		exit(1);
	}

	if (argc == 3 && atoi(argv[2]) < 1)
	{
		printf("Usage: %s [-H] <file> <line>\n", argv[0]);
		printf("<line> must be an integer greater than 0.\n");
		exit(1);
	}

	if (ftype(argv[1]) == _NONE)
	{
		printf("%s: %s: No such file or directory\n", argv[0], argv[1]);
		exit(1);
	} else if (ftype(argv[1]) == _DIR) {
		printf("%s: %s: Is a directory\n", argv[0], argv[1]);
		exit(1);
	}

	FILE *fptr;
	char tfile[128];
	char buffer[1024];
	int line = 0;
	int ok = 0;

	strcpy(tfile, argv[1]);

	fptr = fopen(tfile, "r");
   
	if (!fptr) {
		printf("%s: %s: Could not open file for reading\n", argv[0], tfile);
		exit(1);
	}

	rewind(fptr);

	while(1)
	{
		line++;
		fgets(buffer, 776, fptr);
		if (feof(fptr)) {
			break;
		}
		if (line == atoi(argv[2]))
		{
			printf("%s", buffer);
			ok++;
			break;
		}
	}
	
	fclose(fptr);
	
	if (!ok)
	{
		printf("%s: %s: Line number out of range (1-%d)\n", argv[0], tfile, line - 1);
		exit(1);
	}

	exit(0);
}
