/* Search and Destroy Utility
 *
 * Version 1.0
 * 12/19/2020
 *
 * snd.c
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>

#define PROG_HELP \
"Search and Destroy Utility: Find and delete files with a specific name\n\
\n\
snd [-H] [-v] <file>\n\
\n\
\t[-H] - Display program syntax and options\n\
\t[-v] - Verbose mode; output details of operation\n\
\n\
\t<file> must be the exact name of the file to delete.\n\
\n\
\tCAUTION: This program deletes matching files without confirmation.\n"

#define OPT_HELP	'H'
#define OPT_VERBOSE	'v'

#define FIND_COMMAND	"find / -iname \"%s\" -type f -print 2>/dev/null > %s"

char options[] = "";
char temp_name[L_tmpnam];

enum
{
	_NONE,
	_FILE,
	_DIR
};

int inopts(char opt)
{
	static int sr;
	sr = 0;
	int z;

	for (z = 0; z <= strlen(options); z++)
	{
		if (options[z] == opt)
		{
			sr = 1;
			break;
		} else {
			continue;
		}
	}

	return sr;
}

int ftype(char file[])
{
	struct stat fstat;
	int s = stat(file, &fstat);
	if (s != 0)
	{
		return _NONE;
	}
	if (S_ISREG (fstat.st_mode)) 
	{
		return _FILE;
	} else if (S_ISDIR (fstat.st_mode)) 
	{
		return _DIR;
	} else {
		return _NONE;
	}
}

int empty_file(char tfile[])
{
	if (ftype(tfile) == _NONE)
	{
		return 1;
	}
	
	struct stat file;
	stat(tfile, &file);
	int _fsize = file.st_size;
	if (_fsize == 0)
	{
		return 1;
	} else {
		return 0;
	}
}

char * set_temp_file(char pattern[])
{
	char cmd[128];
	FILE *fptr;

	strcpy(temp_name, "/tmp/TmpXXXXXX");

	mkstemp(temp_name);

	fptr = fopen(temp_name, "w");

	if (!fptr)
	{
		return "(file access error)";
	}

	fclose(fptr);

	sprintf(cmd, FIND_COMMAND, pattern, temp_name);

	system(cmd);

	return "OK";
}

int del_temp_file(void)
{
	if (remove(temp_name) == 0)
	{
		return 0;
	} else {
		return 1;
	}
}

int main(int argc, char *argv[])
{
	int d = 0;
	int p = 1;
	int r;
	char line[64];
	FILE *fptr;

	if (argc >= 2 && argv[1][0] == '-')
	{
		p = 2;

		for (r = 1; r < strlen(argv[1]); r++)
		{
			options[r - 1] = argv[1][r];
		}
	}

	if (inopts(OPT_HELP))
	{
		printf(PROG_HELP);
		exit(0);
	}

	if (argc < 2 || (argc == 2 && argv[1][0] == '-'))
	{
		printf("Usage: %s [-H] [-v] <file-name>\n", argv[0]);
		exit(1);
	}

	set_temp_file(argv[p]);

	if (empty_file(temp_name))
	{
		printf("%s: No matches\n", argv[0]);
		del_temp_file();
		exit(1);
	}

	fptr = fopen(temp_name, "r");

	while (1)
	{
		if (feof(fptr))
		{
			break;
		}

		fscanf(fptr, "%s\n", line);

		if (ftype(line) == _DIR)
		{
			printf("%s: %s is a directory\n", argv[0], line);
			continue;
		}

		if (!remove(line))
		{
			if (inopts(OPT_VERBOSE))
			{
				printf("Deleted '%s'\n", line);
				d++;
			}
		} else {
				printf("%s: Failed to delete '%s' (%s)\n", argv[0], line, strerror(errno));
		}

	}

		fclose(fptr);

		if (del_temp_file() != 0)
		{
			printf("%s: Failed to delete temporary file %s\n", argv[0], temp_name);
		}

		if (inopts(OPT_VERBOSE))
		{
			printf("%s: %d files deleted\n", argv[0], d);
		}

	exit(0);
}
